@@ -643,7 +643,11 @@ static void add_to_vbox_verify_fingerprint(GtkWidget *vbox, ConnContext *context
     label = gtk_label_new(NULL);
     
     gtk_label_set_markup(GTK_LABEL(label), label_text);
-    gtk_label_set_selectable(GTK_LABEL(label), FALSE);
+    /* Make the label containing the fingerprints selectable, but
+     * not auto-selected. */
+    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
+    g_object_set(label, "can-focus", FALSE, NULL);
+
     g_free(label_text);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
@@ -2173,6 +2177,12 @@ static void conversation_destroyed(PurpleConversation *conv, void *data)
     otrg_gtk_dialog_free_smp_data(conv);
 
     gtkconv = PIDGIN_CONVERSATION ( conv );
+
+    /* Only delete the OTR menus if we're the active conversation */
+    if (gtkconv != pidgin_conv_window_get_active_gtkconv(gtkconv->win)) {
+	return;
+    }
+
     win = pidgin_conv_get_window ( gtkconv );
     menu_list = g_hash_table_lookup ( otr_win_menus, win );
     iter = menu_list;
