@@ -1,6 +1,6 @@
 /*
  *  Off-the-Record Messaging plugin for pidgin
- *  Copyright (C) 2004-2008  Ian Goldberg, Rob Smits,
+ *  Copyright (C) 2004-2012  Ian Goldberg, Rob Smits,
  *                           Chris Alexander, Nikita Borisov
  *                           <otr@cypherpunks.ca>
  *
@@ -155,6 +155,9 @@ void otrg_plugin_create_privkey(const char *accountname,
 	const char *protocol)
 {
     OtrgDialogWaitHandle waithandle;
+#ifndef WIN32
+    mode_t mask;
+#endif  /* WIN32 */
     FILE *privf;
 
     gchar *privkeyfile = g_build_filename(purple_user_dir(), PRIVKEYFNAME, NULL);
@@ -162,7 +165,13 @@ void otrg_plugin_create_privkey(const char *accountname,
 	fprintf(stderr, _("Out of memory building filenames!\n"));
 	return;
     }
+#ifndef WIN32
+    mask = umask (0077);
+#endif  /* WIN32 */
     privf = g_fopen(privkeyfile, "w+b");
+#ifndef WIN32
+    umask (mask);
+#endif  /* WIN32 */
     g_free(privkeyfile);
     if (!privf) {
 	fprintf(stderr, _("Could not write private key file\n"));
@@ -287,7 +296,7 @@ static void still_secure_cb(void *opdata, ConnContext *context, int is_reply)
 
 static void log_message_cb(void *opdata, const char *message)
 {
-    purple_debug_info("otr", message);
+    purple_debug_info("otr", "%s", message);
 }
 
 static int max_message_size_cb(void *opdata, ConnContext *context)
@@ -619,9 +628,18 @@ void otrg_plugin_disconnect(ConnContext *context)
 /* Write the fingerprints to disk. */
 void otrg_plugin_write_fingerprints(void)
 {
+#ifndef WIN32
+    mode_t mask;
+#endif  /* WIN32 */
     FILE *storef;
     gchar *storefile = g_build_filename(purple_user_dir(), STOREFNAME, NULL);
+#ifndef WIN32
+    mask = umask (0077);
+#endif  /* WIN32 */
     storef = g_fopen(storefile, "wb");
+#ifndef WIN32
+    umask (mask);
+#endif  /* WIN32 */
     g_free(storefile);
     if (!storef) return;
     otrl_privkey_write_fingerprints_FILEp(otrg_plugin_userstate, storef);
@@ -972,6 +990,11 @@ __init_plugin(PurplePlugin *plugin)
     otrg_dialog_set_ui_ops(otrg_gtk_dialog_get_ui_ops());
 #endif
 
+#ifndef WIN32
+    /* Make key generation use /dev/urandom instead of /dev/random */
+    gcry_control(GCRYCTL_ENABLE_QUICK_RANDOM, 0);
+#endif
+
     /* Initialize the OTR library */
     OTRL_INIT;
 
